package handler

import (
	"net/http"

	"git.frostfs.info/TrueCloudLab/frostfs-observability/tracing"
	"git.frostfs.info/TrueCloudLab/frostfs-s3-gw/api/middleware"
)

func (h *handler) GetBucketLocationHandler(w http.ResponseWriter, r *http.Request) {
	ctx, span := tracing.StartSpanFromContext(r.Context(), "handler.GetBucketLocation")
	defer span.End()

	reqInfo := middleware.GetReqInfo(ctx)

	bktInfo, err := h.getBucketAndCheckOwner(r, reqInfo.BucketName)
	if err != nil {
		h.logAndSendError(ctx, w, "could not get bucket info", reqInfo, err)
		return
	}

	if err = middleware.EncodeToResponse(w, LocationResponse{Location: bktInfo.LocationConstraint}); err != nil {
		h.logAndSendError(ctx, w, "couldn't encode bucket location response", reqInfo, err)
	}
}
